import os
os.chdir('/home/alvaro/tensorflow/models/research/object_detection') # Set working dir

import numpy as np
import six.moves.urllib as urllib
import sys
import signal
import time
import tarfile
import tensorflow as tf
import zipfile

from collections import defaultdict
from io import StringIO
from matplotlib import pyplot as plt
from PIL import Image

from object_detection.utils import label_map_util
from object_detection.utils import visualization_utils as vis_util

from argparse import ArgumentParser

parser = ArgumentParser()
#required = parser.add_argument_group('required arguments')
parser.add_argument("-i", "--input", dest="input", help="path to the analyzed image", metavar="FILE")
parser.add_argument("-o", "--output", dest="output", help="filename of the generated detected image", metavar="FILE")
parser.add_argument("-p", "--ppid", dest="ppid", help="Parent Proccess ID")
parser.add_argument("-s", "--simple", dest="simple", help="generate simple output", action='store_true')
args = parser.parse_args()
input_image = args.input;
output_image = args.output;
ppid = int(args.ppid);
simple_output = args.simple;

	
# Function definitions
def load_image_into_numpy_array(image):
	(im_width, im_height) = image.size
	return np.array(image.getdata()).reshape(
		(im_height, im_width, 3)).astype(np.uint8)

def detect_objects(image_path):
	with detection_graph.as_default():
		#Deprecated
		#with tf.Session(graph=detection_graph) as sess:
			#sess.run(tf.global_variables_initializer())
		
		with tf.compat.v1.Session(graph=detection_graph) as sess:
			sess.run(tf.compat.v1.global_variables_initializer())
			   
			image = Image.open(image_path)
			image_np = load_image_into_numpy_array(image)
			# Expand dimensions since the model expects images to have shape: [1, None, None, 3]
			image_np_expanded = np.expand_dims(image_np, axis=0)
			image_tensor = detection_graph.get_tensor_by_name('image_tensor:0')
			# Each box represents a part of the image where a particular object was detected.
			boxes = detection_graph.get_tensor_by_name('detection_boxes:0')
			scores = detection_graph.get_tensor_by_name('detection_scores:0')
			classes = detection_graph.get_tensor_by_name('detection_classes:0')
			num_detections = detection_graph.get_tensor_by_name('num_detections:0')

			(boxes, scores, classes, num_detections) = sess.run(
				[boxes, scores, classes, num_detections],
				feed_dict={image_tensor: image_np_expanded})

			vis_util.visualize_boxes_and_labels_on_image_array(
				image_np,
				np.squeeze(boxes),
				np.squeeze(classes).astype(np.int32),
				np.squeeze(scores),
				category_index,
				use_normalized_coordinates=True,
				line_thickness=8)

			plt.figure(figsize=IMAGE_SIZE)
			plt.imsave('RESULTS/output_image.jpg', image_np)            
			plt.imsave('/home/alvaro/output_image.jpg', image_np)

			# Write found objects
			if simple_output:
				f = open('/home/alvaro/output_classes.txt', 'w')
				print("# CLASSES #")
				f.write("# CLASSES #\n")
				first_it = True
				classes_output = ([category_index.get(value) for index,value in enumerate(classes[0]) if scores[0,index] > 0.45])
				for class_output in classes_output:
					print(class_output['name'])
					if first_it == True:
						f.write(class_output['name'])
						first_it = False
					else:
						f.write("\n" + class_output['name'])
				f.close()
				#os.chmod('RESULTS/output_classes.txt', 0664) # Not necessary -> DIR has setgid bit set

			else:
				print('\n\n')
				print('#########################################################')
				print('#####################   RESULTS   #######################')
				print('#########################################################')
				print('Total detections: {}'.format(num_detections))
				print('#########################################################')
				#print('score >= 0')
				#print([category_index.get(value) for index,value in enumerate(classes[0]) if scores[0,index] >= 0])
				#print('#########################################################')
				print('score > 50%\n')  
				print([category_index.get(value) for index,value in enumerate(classes[0]) if scores[0,index] > 0.5])
				print('#########################################################')
				#print('score > 75%')
				#print([category_index.get(value) for index,value in enumerate(classes[0]) if scores[0,index] > 0.75])
				print('#########################################################')
				print('\n\n')

# Signal Handling
def signal_handler_SIGUSR1(sig, frame):
	print("#####################################################")
	print("Signal SIGUSR1 received")
	# Detect Objects by calling the function
	print("Calling 'detect_objects(\"/home/alvaro/record.jpg\")'")
	print("-----------------------------------------------------")
	detect_objects("/home/alvaro/record.jpg")
	# Objects have been detected and output_classes written to the destination file
	print("-----------------------------------------------------")
	print("Success!")
	print("Output files have been generated")
	print("#####################################################")
	# I'm ready again (Send Signal to 'script.py')
	os.kill(ppid, signal.SIGUSR2)
	print("\n> Ready!\n")
	
signal.signal(signal.SIGUSR1, signal_handler_SIGUSR1)

PATH_TO_CKPT = 'ssdlite_mobilenet_v2_coco_2018_05_09/frozen_inference_graph.pb'
#PATH_TO_CKPT = 'ssd_mobilenet_v1_ppn_shared_box_predictor_300x300_coco14_sync_2018_07_03/frozen_inference_graph.pb'

PATH_TO_LABELS = 'data/mscoco_label_map.pbtxt'

NUM_CLASSES = 90

detection_graph = tf.Graph()
with detection_graph.as_default():
	#od_graph_def = tf.GraphDef()
	#with tf.gfile.GFile(PATH_TO_CKPT, 'rb') as fid:
	od_graph_def = tf.compat.v1.GraphDef()
	with tf.io.gfile.GFile(PATH_TO_CKPT, 'rb') as fid:
		serialized_graph = fid.read()
		od_graph_def.ParseFromString(serialized_graph)
		tf.import_graph_def(od_graph_def, name='')

label_map = label_map_util.load_labelmap(PATH_TO_LABELS)
categories = label_map_util.convert_label_map_to_categories(label_map, max_num_classes=NUM_CLASSES, use_display_name=True)
category_index = label_map_util.create_category_index(categories)

#PATH_TO_TEST_IMAGES_DIR = 'test_images/'
#TEST_IMAGE_PATHS = [ os.path.join(PATH_TO_TEST_IMAGES_DIR, 'image{}.jpg'.format(i)) for i in range(1, 4) ]
#print(TEST_IMAGE_PATHS);

IMAGE_SIZE = (12, 8)

# All resources loaded and ready
print("\n> Ready [Waiting 'SIGUSR1']\n")

# Send signal to 'script.py' when I'm ready
os.kill(ppid, signal.SIGUSR2)

# Polling loop
while True:
	time.sleep(1) # Wait until signal

#END!
	 